package com.lishihe.hosts.frame;

import com.lishihe.hosts.entity.constant.HostsConstant;
import com.lishihe.hosts.utils.FileUtils;
import com.lishihe.hosts.utils.PlugsUtils;
import org.apache.commons.lang3.StringUtils;

import javax.accessibility.AccessibleContext;
import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;
import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.net.URL;
import java.util.*;
import java.util.List;
import java.util.stream.Collectors;

public class HostsFrame {
    //页面数据
    //--系统环境是linux？
    private Boolean linuxIs;
    //--文件名和文件路径Map
    private Map<String, String> fileNameAndPathMap;
    //--表格被选中的行
    private int[] tableRowsSelected;
    //--表格被选中的列
    private int[] tableCellsSelected;
    //--model
    private DefaultTableModel defaultTableModel;

    //页面组件
    //--主面板
    private JPanel mainPanel;
    //--主面板的分割面板 分为左右两部分
    private JSplitPane splitPane;
    //--左侧部分 文件名列表
    private JList<String> hostNameList;
    //--右侧部分 文件内容
    private JTable hostTable;
    //--保存按钮
    private JButton btn_save;
    //--文件名编辑框
    private JTextField textInput;
    //--文件名列表右键菜单
    private JPopupMenu nameListMenu;
    //--表格右键菜单
    private JPopupMenu nameTableMenu;

    public HostsFrame() {
        //初始化
        init();
        //初始化左侧host文件列表
        initHostList();
        //初始化右侧host文件配置项
        initHostTable(null);
        //初始化按钮监听
        initBtn();
    }

    private void initBtn() {
        btn_save.addActionListener(eventE -> {
            String newFileName = textInput.getText();
            if (StringUtils.isBlank(newFileName)) {
                PlugsUtils.modalMessageError("请输入文件名称~");
                return;
            }
            String oldFileName = getSelected();
            if (StringUtils.isBlank(oldFileName)) {
                PlugsUtils.modalMessageError("请先选中目标文件~");
                return;
            }
            //更新正文
            StringJoiner content = new StringJoiner("\n");
            for (int i = 0; i < hostTable.getRowCount(); i++) {
                content.add(hostTable.getValueAt(i, 0) + "\t\t\t\t" + hostTable.getValueAt(i, 1));
            }
            String hosts = parseHostsFileAndPath(oldFileName);
            FileUtils.writeInFile(hosts, content.toString());
            //更新文件名
            if (!oldFileName.equals(newFileName)) {
                if (fileNameAndPathMap.containsKey(newFileName)) {
                    PlugsUtils.modalMessageError("文件名已存在，换一个再试试吧");
                } else {
                    FileUtils.singleChange(new File(hosts), new File(parseHostsFileAndPath(newFileName)));
                    //重新拉取列表
                    parseHostList();
                }
            }
            PlugsUtils.modalMessageInfo("操作成功~");
        });
    }

    /**
     * 初始化
     */
    private void init() {
        //初始化系统环境标识
        linuxIs = "linux".equalsIgnoreCase(System.getProperty("os.name"));
        //初始化工作文件夹
        if (!linuxIs) {
            //--windows D盘申请操作目录
            //--windows 对于c盘文件一般用户操作不了
            //--linux 对于etc文件一般用户操作不了
            FileUtils.checkDirectory(new File(HostsConstant.Systems.WIN_D_DIR_AREA_LOCATION));
        }
    }

    /**
     * 如果发现文件夹内没有hosts文件的备份文件就立刻新增备份
     * 1.防止误操作之后无法还原过程复杂的问题
     */
    private void backSystemHostsFile() {
        String hosts = parseHostsFileAndPath(null);
        String back_hosts = parseHostsFileAndPath(HostsConstant.Systems.MAIN_HOSTS_FILE_BACK_NAME);
        //将系统内的host文件进行备份
        FileUtils.copyFileUsingFileChannels(new File(hosts), new File(back_hosts));
    }

    /**
     * 目标文件事件联动
     *
     * @param name 目标源文件
     */
    private void initListMenu(String name) {
        nameListMenu = new JPopupMenu();
        if (!HostsConstant.Systems.MAIN_HOSTS_FILE_NAME.equals(name)) {
            //获取hosts文件地址名称
            String hosts = parseHostsFileAndPath(null);
            //1
            JMenuItem itemInUse = new JMenuItem("应用为系统环境", getMenuIcon("set"));
            //--设置监听方法  匿名类
            itemInUse.addActionListener(itemE -> {
                //文件覆盖
                FileUtils.writeInFile(hosts, findFileContext(name));
            });
            nameListMenu.add(itemInUse);
            //2
            JMenuItem itemRename = new JMenuItem("重命名", getMenuIcon("total"));
            itemRename.addActionListener(itemE -> {
                PlugsUtils.modalMessageInfo("在右侧文件名输入框内进行操作吧~");
            });
            nameListMenu.add(itemRename);
            //3
            JMenuItem itemDelete = new JMenuItem("删除", getMenuIcon("delete"));
            itemDelete.addActionListener(itemE -> {
                //文件删除  |   c盘文件删不掉
                FileUtils.delete(parseHostsFileAndPath(name));
                //重新拉取列表
                parseHostList();
                //重置右侧
                emptyTable();
            });
            nameListMenu.add(itemDelete);
        }
        //4  文件新增  windows 新增到工作目录，其他系统是hosts文件并列
        JMenuItem itemNew = new JMenuItem("新增一条配置", getMenuIcon("add"));
        itemNew.addActionListener(itemE -> {
            FileUtils.newFile(HostsConstant.Systems.WIN_D_DIR_AREA_LOCATION.concat("\\".concat(name) + (int) (Math.random() * 1000 + 1)));
            //重新拉取列表
            parseHostList();
        });
        nameListMenu.add(itemNew);
        //5 复制文件路径到系统剪切板
        JMenuItem itemCopyPath = new JMenuItem("复制路径", getMenuIcon("copy"));
        itemCopyPath.addActionListener(itemE -> {
            Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
            clip.setContents(new StringSelection(fileNameAndPathMap.get(name)), null);
        });
        nameListMenu.add(itemCopyPath);
        //6 刷新
        JMenuItem itemReload = new JMenuItem("刷新", getMenuIcon("reload"));
        itemReload.addActionListener(itemE -> {
            //重新拉取列表
            parseHostList();
        });
        nameListMenu.add(itemReload);
    }

    /**
     * 目标文件事件联动
     */
    private void initTableMenu() {
        nameTableMenu = new JPopupMenu();
        //1
        JMenuItem itemPush = new JMenuItem("新增一行", getMenuIcon("scan"));
        //--设置监听方法  匿名类
        itemPush.addActionListener(itemE -> {
            defaultTableModel.addRow(new String[]{"", ""});
        });
        nameTableMenu.add(itemPush);
        //2
        JMenuItem itemInsert = new JMenuItem("插入一行", getMenuIcon("add"));
        //--设置监听方法  匿名类
        itemInsert.addActionListener(itemE -> {
            if (tableCellsSelected.length <= 0) {
                return;
            }
            defaultTableModel.insertRow(tableRowsSelected[tableRowsSelected.length - 1], new String[]{"", ""});
        });
        nameTableMenu.add(itemInsert);
        //2
        JMenuItem itemDelete = new JMenuItem("删除", getMenuIcon("delete"));
        //--设置监听方法  匿名类
        itemDelete.addActionListener(itemE -> {
            if (tableCellsSelected.length <= 0) {
                return;
            }
            for (int i = 0; i <= tableCellsSelected[tableCellsSelected.length - 1] - tableCellsSelected[0]; i++) {
                defaultTableModel.removeRow(tableRowsSelected[0]);
            }
        });
        nameTableMenu.add(itemDelete);
    }

    /**
     * 获取图标对象
     *
     * @param fileName 图标名称
     * @return 图标对象
     */
    private ImageIcon getMenuIcon(String fileName) {
        URL resource = getClass().getResource("/static/" + fileName + ".png");
        if (resource != null) {
            ImageIcon logo = new ImageIcon(resource);
            Image oldLogo = logo.getImage().getScaledInstance(20, 20, Image.SCALE_SMOOTH);
            return new ImageIcon(oldLogo);
        }
        return null;
    }

    /**
     * 初始化右侧内容
     *
     * @param fileName 文件名称
     */
    private void initHostTable(String fileName) {
        defaultTableModel = new DefaultTableModel(findFileDetails(fileName), new String[]{"IP", "映射值"});
        initTableMenu();
        hostTable.setModel(defaultTableModel);
        hostTable.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
                if (e.getButton() == 3) {
                    System.err.println(Arrays.toString(hostTable.getSelectedRows()));
                    //得到选中的行列的索引值
                    tableRowsSelected = hostTable.getSelectedRows();
                    tableCellsSelected = hostTable.getSelectedColumns();
                    nameTableMenu.show(e.getComponent(), e.getX(), e.getY());
                    super.mouseClicked(e);
                }
            }
        });
    }

    private void emptyTable() {
        hostTable.setModel(new DefaultTableModel());
        textInput.setText(null);
    }

    /**
     * 获取文件内容
     *
     * @param fileName 文件名
     * @return 文件正文
     */
    private String[][] findFileDetails(String fileName) {
        String[][] result = new String[0][1];
        BufferedReader reader = null;
        try {
            reader = new BufferedReader(new FileReader(parseHostsFileAndPath(fileName)));
            result = reader.lines()
                    .filter(f -> !f.startsWith("#") && StringUtils.isNotBlank(f))
                    .map(m -> m.split("\\s+"))
                    .toArray(String[][]::new);
            reader.close();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (reader != null) {
                try {
                    reader.close();
                } catch (IOException e) {
                    System.out.println(e.getMessage());
                }
            }
        }
        if (result.length == 0 || result.length == 1 && result[0].length == 0) {
            result = new String[][]{{"", ""}};
        }
        return result;
    }

    /**
     * 获取写入应用的正文
     * 1.所有的 #开头的注释行，对于这里是不起作用的
     *
     * @param fileName 文件名称，去找目标文件的
     * @return 正文
     */
    private String findFileContext(String fileName) {
        BufferedReader reader = null;
        String result = "";
        try {
            reader = new BufferedReader(new FileReader(parseHostsFileAndPath(fileName)));
            result = reader.lines()
                    .filter(f -> !f.startsWith("#") && StringUtils.isNotBlank(f))
                    .collect(Collectors.joining("\n"));
            reader.close();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (reader != null) {
                try {
                    reader.close();
                } catch (IOException e) {
                    System.out.println(e.getMessage());
                }
            }
        }
        return result;
    }


    /**
     * 根据文件名称组全路径和名称
     *
     * @param fileName 文件名称
     * @return 文件路径文件名称
     */
    private String parseHostsFileAndPath(String fileName) {
        if (fileName == null) {
            fileName = "hosts";
        }
        return fileNameAndPathMap.getOrDefault(fileName, linuxIs ? "/etc/".concat(fileName) : HostsConstant.Systems.WIN_D_DIR_AREA_LOCATION.concat("\\".concat(fileName)));
    }

    /**
     * 获取hosts文件所属位置的文件夹路径
     *
     * @return 文件夹路径
     */
    private String parseHostsPath() {
        String hostLocation;
        if (linuxIs) {
            hostLocation = "/etc";
        } else {
            hostLocation = System.getenv("windir") + "\\system32\\drivers\\etc";
        }
        return hostLocation;
    }

    /**
     * 初始化配置文件列表
     */
    private void initHostList() {
        parseHostList();
        //设置监听:选项被选中
        hostNameList.addListSelectionListener(listSelectionEvent -> {
            if (listSelectionEvent.getValueIsAdjusting()) {
                // 输出选中的选项
                String selected = getSelected();
                initHostTable(selected);
                textInput.setText(selected);
            } else {
                //松手
                System.out.println("松手");
            }
        });
        hostNameList.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
                // 1.2.3.
                if (e.getButton() == 3) {
                    //初始化左侧右键菜单  |   被选中的文件不能是系统正在使用的文件，也就是hosts文件
                    String selectedFileName = getSelected();
                    if (StringUtils.isNotBlank(selectedFileName)) {
                        initListMenu(selectedFileName);
                        nameListMenu.show(e.getComponent(), e.getX(), e.getY());
                    }
                }
            }
        });
    }

    private void parseHostList() {
        //示例数据
        hostNameList.setListData(findHostNameList());
    }

    private String getSelected() {
        //按下
        int[] indices = hostNameList.getSelectedIndices();
        if (indices.length <= 0) {
            return null;
        }
        // 获取选项数据的 ListModel
        ListModel<String> listModel = hostNameList.getModel();
        return listModel.getElementAt(indices[0]);
    }

    /**
     * 读取文件列表
     *
     * @return 符合条件的文件列表
     */
    private String[] findHostNameList() {
        fileNameAndPathMap = new HashMap<>(100);
        List<String> collect = parseFiles(parseHostsPath());
        if (!linuxIs) {
            collect.addAll(parseFiles(HostsConstant.Systems.WIN_D_DIR_AREA_LOCATION));
        }
        return collect.toArray(new String[0]);
    }

    /**
     * 组装路径下文件列表
     *
     * @param hostLocation 文件路径
     * @return 文件列表
     */
    private List<String> parseFiles(String hostLocation) {
        File dirFile = new File(hostLocation);
        if (dirFile.exists() && dirFile.isDirectory()) {
            File[] files = dirFile.listFiles();
            if (files != null && files.length > 0) {
                return Arrays.stream(files)
                        .map(File::getName)
                        .filter(f -> StringUtils.isNotBlank(f)
                                && f.startsWith(HostsConstant.Systems.MAIN_HOSTS_FILE_NAME)
                                && !f.endsWith(HostsConstant.Systems.MAIN_HOSTS_FILE_NAME_NOT_SHOW))
                        .peek(p -> fileNameAndPathMap.put(p, hostLocation.concat("\\" + p)))
                        .collect(Collectors.toList());
            }
        }
        return new ArrayList<>();
    }

    /**
     * 获取主框用于主显工厂获取面板
     *
     * @return 主面板
     */
    public JPanel getMainPanel() {
        return mainPanel;
    }
}
