package com.lishihe.hosts.utils;

import org.apache.commons.lang3.StringUtils;

import java.io.*;
import java.nio.channels.FileChannel;
import java.nio.charset.StandardCharsets;
import java.util.List;


public class FileUtils {

    /**
     * 覆盖写入
     *
     * @param filePath 目标文件路径
     * @param content  需要写入的内容
     */
    public static void writeInFile(String filePath, String content) {
        File file = new File(filePath);
        if (!file.exists()) {
            try {
                boolean newFile = file.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        try {
            FileOutputStream fos = new FileOutputStream(file);
            OutputStreamWriter osw = new OutputStreamWriter(fos, StandardCharsets.UTF_8);
            osw.write(content);
            osw.flush();
            osw.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 文件拷贝
     *
     * @param source 源文件
     * @param dist   目的文件
     */
    @SuppressWarnings({"resource"})
    public synchronized static void copyFileUsingFileChannels(File source, File dist) {
        checkFile(dist);
        FileChannel inputChannel = null;
        FileChannel outputChannel = null;
        try {
            inputChannel = new FileInputStream(source).getChannel();
            outputChannel = new FileOutputStream(dist).getChannel();
            outputChannel.transferFrom(inputChannel, 0, inputChannel.size());
        } catch (Exception e) {
            System.err.println(e.getMessage());
        } finally {
            assert inputChannel != null;
            assert outputChannel != null;
            try {
                inputChannel.close();
                outputChannel.close();
            } catch (IOException e) {
                System.err.println(e.getMessage());
            }
        }
    }

    /**
     * <p>Title: clearInfoForFile</p>
     * <p>Description: 文件内容清空</p>
     *
     * @param fileName 文件名称
     */
    public static void clearInfoForFile(String fileName) {
        if (StringUtils.isEmpty(fileName)) {
            return;
        }
        File file = new File(fileName);
        try {
            if (!file.exists()) {
                boolean newFile = file.createNewFile();
            }
            FileWriter fileWriter = new FileWriter(file);
            fileWriter.write("");
            fileWriter.flush();
            fileWriter.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * <p>Title: writeFileByLine</p>
     * <p>Description: 按行向文件写入内容</p>
     */
    public static void writeFileByLine(String fileName, List<String> parameter) {
        if (StringUtils.isEmpty(fileName) || parameter == null || parameter.size() == 0) {
            return;
        }
        FileWriter fw = null;
        try {
            fw = new FileWriter(fileName);
            for (String s : parameter) {
                fw.write(s + "\n");
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                assert fw != null;
                fw.close();
            } catch (IOException e) {
                System.err.println(e.getMessage());
            }
        }
    }


    /**
     * 重命名
     *
     * @param source 目标文件
     * @param dist   结果文件
     */
    public static void singleChange(File source, File dist) {
        if (source.renameTo(dist)) {
            System.out.println("重命名成功");
        }
    }

    /**
     * 文件删除
     *
     * @param source 目标文件
     */
    public static void delete(File source) {
        source.deleteOnExit();
    }

    /**
     * 文件删除
     *
     * @param filePath 文件路径
     */
    public static void delete(String filePath) {
        if (new File(filePath).delete()) {
            System.out.println("删除成功！");
        }
    }

    /**
     * 创建文件
     *
     * @param filePath 文件路径
     * @return 文件对象
     */
    public static File newFile(String filePath) {
        File file = new File(filePath);
        checkFile(file);
        return file;
    }

    /**
     * 文件不存在就创建
     *
     * @param file 文件对象
     */
    public static void checkFile(File file) {
        if (!file.exists()) {
            try {
                boolean newFile = file.createNewFile();
            } catch (IOException e) {
                System.err.println(e.getMessage());
            }
        }
    }

    /**
     * 文件夹不存在就创建
     *
     * @param file 文件夹对象
     */
    public static void checkDirectory(File file) {
        if (!file.exists()) {
            try {
                boolean newFile = file.mkdirs();
            } catch (Exception e) {
                System.err.println(e.getMessage());
            }
        }
    }

}
